# Export the members by specific property of a given group 
#
# 20130823 1.0  Initiale Version
# 20140408 1.1  Input File addiert

param (
	$ldapserver		= "LDAP://192.168.23.130/",
	$groupdn		= "CN=syncgroup3,OU=groupsync,DC=msxfaq,DC=local",
	$ldapuser		= "MSXFAQ\Administrator",   # set to "$null" to use integrated authentication
	$ldappass		= "Password!",		# set to $null to use integrated auth
	$ldapauthtype	= "Secure",   # AuthenticationTypes  http://msdn.microsoft.com/de-de/library/system.directoryservices.authenticationtypes.aspx
	$property		= "mail", 
	$outfile        = ""
)


write-host "Export-groupmember:Start"
set-psdebug -strict

$transcriptfile = "GroupSync.Export."+(get-date -format yyyyMMddhh)+".log"
write-host "Export-groupmember:Transcript to:$transcriptfile"
start-transcript -append -path $transcriptfile  | out-null
write-host "Export-groupmember:remove old transcripts"
foreach ($logfile in (Get-ChildItem "groupsync.Export.*.log" | where {$_.lastwritetime -lt (get-date).adddays(-5)} )) {
	write-host "Export-groupmember:remove old transcript: $logfile"
	Remove-Item $logfile
}

write-host "Export-groupmember:Binding Group: $ldapserver$groupdn"
#$group = [adsi]($ldapserver+$groupdn)
$group = new-object System.DirectoryServices.DirectoryEntry(($ldapserver+$groupdn),$ldapuser,$ldappass,$ldapauthtype)

#set the inital from value
$from = 0; $step = 1000; $all = $false 
#array for the members of the group
$members = @() 
write-host "Export-groupmember: GroupDN: $ldapserver$Groupdn"

write-host "Export-groupmember: Initialize Directory Searcher"
$searcher = New-Object DirectoryServices.DirectorySearcher($Group)
$searcher.Searchscope = 'Base'

 while (! $all) { 
	$to = $from + $step -1 # windows 2000 send only 1000 members. win2003+ sends 1500. so be compatible
	write-host "Export-Groupmember: Search from $from  to $to"
	#Query the group object for members using "member;range=$from-$to" to just return the range of objects for this pass.
	$searcher.PropertiesToLoad.Clear() | Out-Null
	$searcher.PropertiesToLoad.Add( "member;Range=$from-$to" ) | Out-Null
	$result = $searcher.FindOne()
	$all = $true
	foreach ($resultproperty in $result.properties.propertynames) {
		if ($resultproperty.startswith("member;")) {
			$all = $false
			write-host "Export-groupmember: Processing $resultproperty"
			foreach ($entry in $result.properties.($resultproperty) ){
				$from += 1
				write-host "Export-groupmember: FoundDN $from : $entry"
				$properyvalue = (new-object System.DirectoryServices.DirectoryEntry(($ldapserver+$entry),$ldapuser,$ldappass,$ldapauthtype)).($property)
				
				if ($properyvalue -eq $null) {
					write-host "Export-groupmember: Warn: Property $property empty"
				}
				else {
					write-host "Export-groupmember: Adding $property : $properyvalue"
					$members += $properyvalue
				}
			}
		}
		if ($resultproperty.endswith("-*")) {
			write-host "Export-groupmember: End of List reached"
			$all = $true
		}
	}
 } 

$members += "###"
write-host "Export-groupmember: Total members found:"$from
write-host "Export-groupmember: Total members added:"$members.count
 
 if ($outfile -eq "") {
	write-host "Sending Data to STDOUT"
	$members
}
else {
	write-host "Sending Data to File: $outfile"
	$members | out-file $outfile
} 

write-host "Export-groupmember: End"
stop-transcript | out-null