# pending.  transscript mit alten entfernen

# Import members by specific property to a given group 

# 1. get list of current members in hashtable
# 2. get list of user via pipeline
# 3. search for each user. 
#	if foundin hashtable -> remove from hashtable
#   else addot group
# 4. remove members still in hashtable

# 20130823 1.0  Initiale Version
# 20140408 1.1  Input File addiert

param (
	$ldapserver		= "LDAP://192.168.23.130/",
	$groupdn		= "CN=syncgroup4,OU=groupsync,DC=msxfaq,DC=local",
	$searchpath 	= "GC://192.168.23.130/dc=msxfaq,dc=local", #"LDAP://192.168.23.130:3268/",   "GC://server/dc=msxfaq,dc=local
	$ldapuser		= "MSXFAQ\Administrator",   # set to $null to use integrated auth
	$ldappass		= "Password!",             # set to $null to use integrated auth
	$ldapauthtype	= "Secure",   # AuthenticationTypes  http://msdn.microsoft.com/de-de/library/system.directoryservices.authenticationtypes.aspx
	$property		= "mail",
	$resultfile		= ".\groupsync.change.csv",
	$infile         = "" ,
	$removeorphaned	= $false
)

write-host "Import-groupmember:Start"
set-psdebug -strict

$transcriptfile = "GroupSync.Import."+(get-date -format yyyyMMddhh)+".log"
write-host "Import-groupmember:Transcript to:$transcriptfile"
start-transcript -append -path $transcriptfile | out-null
write-host "Import-groupmember:remove old transcripts"
foreach ($logfile in (Get-ChildItem "groupsync.Import.*.log" | where {$_.lastwritetime -lt (get-date).adddays(-5)} )) {
	write-host "Import-groupmember:remove old transcript: $logfile"
	Remove-Item $logfile
}

write-host "Import-groupmember:Initialize ReportFile $resultfile "
if (!(Test-Path $resultfile -PathType leaf)) {
		write-host "write-result: Create CSVFile:$resultfile "
		"""timestamp"",""groupdn"",""memberdn"",""action""" | out-file $resultfile 
}

write-host "Import-groupmember:Initialize statistic"
$summary = New-Object PSObject -Property @{
	TotalOriginal=0
	TotalSource = 0
	TotalAdd = 0
	TotalRemove = 0
	TotalNotFound = 0
	Totalduplicate = 0
}

write-host "Import-groupmember: Loading Userlist from Pipeline"
$importlist = @()

if ($infile -eq "") {
	write-host "Loading Data from STDIN"
	$input | foreach { $importlist += $_ }
}
else {
	write-host "Loading Data from File: $infile"
	$importlist = get-content $infile
} 


if ($importlist[$importlist.count - 1] -ne "###") {
	write-host "Import-groupmember: Importfile not ending with ###. Stop import"
	exit
}

write-host "Import-groupmember: Total Importusers found: $($importlist.count-1)"
$summary.TotalSource = $importlist.count-1


write-host "Import-groupmember:Binding Group: $ldapserver$groupdn"
$group = new-object System.DirectoryServices.DirectoryEntry(($ldapserver+$groupdn),$ldapuser,$ldappass,$ldapauthtype)

write-host "Import-groupmember: Initialize Directory Searcher for GroupMemberSeach"
#$searcher = New-Object DirectoryServices.DirectorySearcher($Group,"(objectClass=*)","member",'Base') 
$searcher = New-Object DirectoryServices.DirectorySearcher($Group)
$searcher.Searchscope = 'Base'

write-host "Import-groupmember: GroupDN: $ldapserver $Groupdn"
#set the initial from value
$from = 0; $step = 1000; $all = $false 
[hashtable]$groupmember=@{}

 while (! $all) { 
	$to = $from + $step -1 # windows 2000 send only 1000 members. win2003+ sends 1500. so be compatible
	write-host "Import-groupmember: Search from $from  to $to"
    #Query the group object for members using "member;range=$from-$to" to just return the range of objects for this pass.
	$searcher.PropertiesToLoad.Clear() | Out-Null
	$searcher.PropertiesToLoad.Add( "member;Range=$from-$to" ) | Out-Null
	$result = $searcher.FindOne()
	$all = $true
	foreach ($resultproperty in $result.properties.propertynames) {
		if ($resultproperty.startswith("member;")) {
			write-host "Import-groupmember: Processing $resultproperty"
			$all = $false
			foreach ($entry in $result.properties.($resultproperty) ){
				$from += 1
				write-host "Import-groupmember: AddingMemberDNtoHash $from : $entry"
				$groupmember.add($entry,$true)
			}
		}
		if ($resultproperty.endswith("-*")) {
			write-host "Import-groupmember: End of List reached"
			$all = $true
		}
	}
} 

$summary.TotalOriginal=$from
write-host "Import-groupmember: Total members found:"$from

write-host "Import-groupmember: Initializing Searcher $searchpath"

$searchroot = new-object System.DirectoryServices.DirectoryEntry($searchpath,$ldapuser,$ldappass,$ldapauthtype)
$searcher = New-Object DirectoryServices.DirectorySearcher($searchroot)
$searcher.Searchscope = 'Subtree'
#$searcher.Searchroot = ""
$searcher.PageSize = 1000
#$searcher.Filter = "(&(mail=*))"
#$searcher.PropertiesToLoad.Add("mail") | Out-Null
#$searcher.PropertiesToLoad.Add("ProxyAddresses") | Out-Null

 
 foreach ($importuser in $importlist) {
	if ($importuser -ne "###") {  
		write-host "Import-groupmember: Processing Import:" $importuser
		$searcher.Filter = "(&("+$property+"="+$importuser+"))"
		write-host "Import-groupmember: Search "$searcher.Filter
		$result = $searcher.findall()
		if ($result.count -eq 0) {
			write-host "Import-groupmember: User not found in target. skipping"
			$summary.TotalNotFound +=1
			(""""+(get-date -format yyyyMMddHHmmss)+""",""$groupdn"",""$entry"",""NotFound""") | out-file $resultfile -append
		}
		elseif ($result.count -ge 2) {
			write-host "Import-groupmember: User not unique. skipping"
			$summary.Totalduplicate += 1
			(""""+(get-date -format yyyyMMddHHmmss)+""",""$groupdn"",""$entry"",""NotUnique""") | out-file $resultfile -append
		}
		else {
			[string]$dn = $result[0].properties.distinguishedname
			write-host "Import-groupmember: User found at " $dn
			
			if ($groupmember.containskey($dn)) {
				write-host "Import-groupmember: User already in group. Done"
				$groupmember.remove($dn)
			}
			else {
				write-host "Import-groupmember: Adding User to group"
				$group.Member.Add($dn)
				$group.setInfo()
				$summary.TotalAdd +=1
				(""""+(get-date -format yyyyMMddHHmmss)+""",""$groupdn"",""$entry"",""ADDED""") | out-file $resultfile -append
			}
		}
	}
}


write-host "Import-groupmember: Checking orphaned users"
foreach ($dn in $groupmember.keys) {
	write-host "Import-groupmember: remove "$dn
	if ($removeorphaned) {
		$group.Member.Remove($dn)
		$group.setInfo()
		(""""+(get-date -format yyyyMMddHHmmss)+""",""$groupdn"",""$entry"",""REMOVE""") | out-file $resultfile -append
	}
	else {
		(""""+(get-date -format yyyyMMddHHmmss)+""",""$groupdn"",""$entry"",""Orphan""") | out-file $resultfile -append
	}
	$summary.TotalRemove +=1
}

write-host "Import-groupmember: Summary"
$summary
stop-transcript | out-null
write-host "Import-groupmember: End"

